xquery version "3.1";

module namespace removal="https://mollenhauer-edition.de/ns/conversions/removal";

(:~
 : Convert Module Removal
 : *******************************************************************************************
 : This module contains functions to strip the TEI from elements that are not needed for text processing.
 :
 : @version 1.4 (2024-04-03)
 : @status stable
 : @author Uwe Sikora, Max Zeterberg
 :)

import module namespace studienausgabe="https://mollenhauer-edition.de/ns/conversions/studienausgabe" at "studienausgabe.xqm";

declare namespace tei = "http://www.tei-c.org/ns/1.0";
declare namespace output="http://www.w3.org/2010/xslt-xquery-serialization";

declare option output:method "xml";

(:~
 : Default return for removal:removal: If $node is an elment node it will be reconstructed and its child nodes will
 : be passed on to removal:removal. If it is not an element, its child nodes will be passed on to removal:removal.
 :
 : @author Max-Ferdinand Zeterberg
 : @author Uwe Sikora
 : @version 1.0
 :
 : @param $node The current node
:)
declare function removal:default-removal($node as node()*)
{
    try {
        element {QName($node/namespace-uri(), $node/name())} {
            $node/@*,
            removal:removal($node/node())
        }
    } catch * {
        removal:removal($node/node())
    }
};

(:~
 : This function deletes the tags of certain elements. Other Elements will be reconstructed, Text will be normalized.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $node The current node
:)
declare function removal:delete-element-tags($nodes as node()*)
{
    for $node in $nodes
    return
        if (local-name($node) = ('date', 'foreign', 'hi', 'name', 'rs')) then
            removal:delete-element-tags($node/node())
        else
            try {
                element {QName($node/namespace-uri(), $node/name())} {
                    $node/@*,
                    removal:removal($node/node())
                }
            } catch * {
                normalize-space($node)
            }
};

(:~
 : This function strips the TEI-Document from all nodes that should not be converted to plain text. It also surrounds
 : tei:seg/@type="margin" and tei:note with whitespace and replaces tei:pause with whitespace.
 :
 : @author Max-Ferdinand Zeterberg
 : @author Uwe Sikora
 : @version 1.0
 :
 : @param $nodes TEI-Document without critical apparatus.
:)
declare function removal:removal($nodes as node()*) as item()*
{
    for $node in $nodes
    return
        typeswitch ($node)
            case text()
                return
                    ($node)
            case processing-instruction()
                return
                    ()
            case comment()
                return
                    ()
            case element(tei:abbr)
                return
                    if ($node/parent::tei:choice) then
                        ()
                    else
                        removal:default-removal($node)
                        case element(tei:anchor)
                return
                    ()
            case element(tei:bibl)
                return
                    if ($node/@type = "commentary") then
                        ()
                    else
                        removal:default-removal($node)
            case element(tei:date)
                return
                    (removal:delete-element-tags($node/node()))
            case element(tei:facsimile)
                return
                    ()
            case element(tei:figDesc)
                return
                    ()
            case element(tei:foreign)
                return
                    (removal:delete-element-tags($node/node()))
            case element(tei:head)
                return
                    if($node/@type = "ToC") then
                        ()
                    else
                        removal:default-removal($node)
            case element(tei:hi)
                return
                    (removal:delete-element-tags($node/node()))
            case element(tei:label)
                return
                    if ($node/@type = 'note_anchor') then
                        ()
                    else
                        removal:default-removal($node)
            case element(tei:name)
                return
                    (removal:delete-element-tags($node/node()))
            case element(tei:note)
                return
                    if ($node/@type = "commentary") then
                        ()
                    else
                        (" ", removal:default-removal($node), " ")
            case element(tei:pause)
                return
                    ' '
            case element(tei:pb)
                return
                    ()
            case element(tei:rs)
                return
                    (removal:delete-element-tags($node/node()))
            case element(tei:seg)
                return
                    if ($node/@type = "eyecatcher") then
                        removal:default-removal($node)
                    else
                        if ($node/@type = "margin") then
                            (" ", removal:default-removal($node), " ")
                        else
                            ()
            case element(tei:teiHeader)
                return
                    ()
            case element(tei:text)
                return
                    if ($node/@type = "commentary") then
                        ()
                    else
                        removal:default-removal($node)
            default
                return
                    removal:default-removal($node)
};

(:~
 : This function strips the TEI-Document from all tei:sic.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $nodes TEI-Document without critical apparatus.
:)
declare function removal:delete-sic($nodes as node()*) as item()*
{

    for $node in $nodes
    return
        typeswitch ($node)
            case text()
                return
                    ($node)
            case element(tei:sic)
                return
                    ()
            default
                return
                    try {
                        element {QName($node/namespace-uri(), $node/name())} {
                            $node/@*,
                            removal:delete-sic($node/node())
                        }
                    } catch * {
                        removal:delete-sic($node/node())
                    }
};

(:~
 : This function strips the TEI-Document from all tei:corr.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $nodes TEI-Document without critical apparatus.
:)
declare function removal:delete-corr($nodes as node()*) as item()*
{

    for $node in $nodes
    return
        typeswitch ($node)
            case text()
                return
                    ($node)
            case element(tei:corr)
                return
                    ()
            default
                return
                    try {
                        element {QName($node/namespace-uri(), $node/name())} {
                            $node/@*,
                            removal:delete-corr($node/node())
                        }
                    } catch * {
                        removal:delete-corr($node/node())
                    }
};

(:~
 : This function strips the TEI-Document from all tei:sic and tei:corr. Except for those elements with tei:quote as ancestor, the content of tei:corr is kept. If tei:quote is ancestor, the content of tei:sic is kept.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $nodes TEI-Document without critical apparatus.
:)
declare function removal:delete-sic-studienausgabe($nodes as node()*) as item()*
{

    for $node in $nodes
    return
        typeswitch ($node)
            case text()
                return
                    ($node)
            case processing-instruction()
                    return
                        ($node)
            case element(tei:choice)
                return
                    if ($node/child::tei:sic) then
                        if ($node/ancestor::tei:quote) then
                            ($node/tei:sic/node())
                        else
                            ($node/tei:corr/node())
                    else
                        ($node)
            default
                return
                    try {
                        element {QName($node/namespace-uri(), $node/name())} {
                            $node/@*,
                            removal:delete-sic-studienausgabe($node/node())
                        }
                    } catch * {
                        removal:delete-sic-studienausgabe($node/node())
                    }
};


(:~
 : Default return for removal:remove-commentary: If $node is an elment node it will be reconstructed and its child nodes will
 : be passed on to removal:remove-commentary. If it is not an element, its child nodes will be passed on to removal:remove-commentary.
 :
 : @author Max-Ferdinand Zeterberg
 : @author Uwe Sikora
 : @version 1.0
 :
 : @param $node The current node
:)
declare function removal:default-remove-commentary($node as node()*)
{
    try {
        element {QName('http://www.tei-c.org/ns/1.0', $node/name())} {
            $node/@*,
            removal:remove-commentary($node/node())
        }
    } catch * {
        removal:remove-commentary($node/node())
    }
};

(:~
 : This function strips the TEI-Document the Werkkommentar.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $nodes TEI-Document.
:)
declare function removal:remove-commentary($nodes as node()*) as node()*
{
    for $node in $nodes
    return
        typeswitch ($node)
            case text()
                return
                    ($node)
            case processing-instruction()
                return ($node)
            case comment()
                return ($node)
            case element(tei:text)
                return
                    if ($node/@type = "commentary") then
                        ()
                    else
                        removal:default-remove-commentary($node)
            default
                return
                    removal:default-remove-commentary($node)
};

(:~
 : Default return for removal:remove-witnesses: If $node is an elment node it will be reconstructed and its child nodes will
 : be passed on to removal:remove-witnesses. If it is not an element, its child nodes will be passed on to removal:remove-witnesses.
 :
 : @author Max-Ferdinand Zeterberg
 : @author Uwe Sikora
 : @version 1.0
 :
 : @param $node The current node
:)
declare function removal:default-remove-witnesses($node as node()*, $fassung as xs:string)
{
    try {
        element {QName($node/namespace-uri(), $node/name())} {
            $node/@*,
            removal:remove-witnesses($node/node(), $fassung)
        }
    } catch * {
        removal:remove-witnesses($node/node(), $fassung)
    }
};


(:~
 : This function strips not used witnesses.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $nodes TEI-Document.
:)
declare function removal:remove-witnesses($nodes as node()*, $fassung as xs:string) as node()*
{
    for $node in $nodes
    return
        typeswitch ($node)
            case text()
                return
                    ($node)
            case processing-instruction()
                return ($node)
            case comment()
                return ($node)
            case element(tei:bibl)
                return
                    if ($node/parent::tei:witness) then
                        studienausgabe:fullreference-zotero($node/@corresp, "")
                    else
                        removal:default-remove-witnesses($node, $fassung)
            case element(tei:witness)
                return
                    if ($node/@xml:id = $fassung) then
                        (element {QName('http://www.tei-c.org/ns/1.0', 'date')}{
                                attribute type {'erstveroeffentlichung'},
                                attribute when {$node/parent::tei:listWit/tei:witness[1]//tei:date[1]/@when}
                            },
                        removal:default-remove-witnesses($node, $fassung))
                    else
                        if ($node/parent::tei:listWit/parent::tei:witness) then
                            removal:default-remove-witnesses($node, $fassung)
                        else
                            ()
            default
                return
                    removal:default-remove-witnesses($node, $fassung)
};
